close all;
clear all;
addpath('../leverageScores');
addpath('..');
% load('BW_canny.mat');

I  = imread('/home/jan/dtu/speciale/images/12.jpg');
Igray = imadjust(rgb2gray(I(1:2:end,1:2:end,:)));
BW = edge(Igray,'canny');

width = size(BW,2);
height = size(BW,1);

windowWidth = 15;
windowHeight = 15;

xWindows = floor(width/windowWidth);
yWindows = floor(height/windowHeight);

%% Full problem
tic;
linearityMatrixFull = zeros(yWindows,xWindows);
leverageScoreHists = NaN(xWindows*yWindows,20);
for x=1:xWindows
    for y=1:yWindows
        edgeSample = BW((1+(y-1)*windowHeight):(y*windowHeight),(1+(x-1)*windowWidth):(x*windowWidth));
        [ yWhite, xWhite ] = find(edgeSample == 1);
        xWhiteOnes = [ ones(length(xWhite),1) xWhite ];
        yWhiteOnes = [ ones(length(yWhite),1) yWhite ];
        h = diag(xWhiteOnes*((xWhiteOnes'*xWhiteOnes) \ xWhiteOnes'));
        if(sum(h) < 10e-6)
            h = (1/length(xWhite))*ones(length(xWhite),1);
        end
        h = h / sum(h);
        if(~isempty(h) && sum(isnan(h)) == 0)
            leverageScoreHists(x*y,:) = hist(h,0.025:0.05:0.975);
            leverageScoreHists(x*y,:) = leverageScoreHists(x*y,:) / sum(leverageScoreHists(x*y,:));
        end
        linearity = 10;
        if(length(xWhite) >= 5)
            beta = ((xWhiteOnes'*xWhiteOnes) \ xWhiteOnes')*yWhite;
            yHat = xWhiteOnes*beta;
            residuals = abs(yWhite - yHat);
            linearityX = mean(residuals);
            
            beta = ((yWhiteOnes'*yWhiteOnes) \ yWhiteOnes')*xWhite;
            xHat = yWhiteOnes*beta;
            residuals = abs(xWhite - xHat);
            linearityY = nanmean(residuals);
            linearity = nanmin(linearityX,linearityY);
        end
        linearityMatrixFull(y,x) = linearity;
    end
end
linearityMatrixFull = linearityMatrixFull / nanmax(linearityMatrixFull(:));
thresholdedLinearityMatrixFull = linearityMatrixFull<0.4*nanmean(linearityMatrixFull(:));
compTimeFull = toc;
figure, imagesc(thresholdedLinearityMatrixFull), colormap gray;
figure, imagesc(linearityMatrixFull), colormap gray;

positives = sum(thresholdedLinearityMatrixFull(:));

reportPlot('bar','Average leverage score histogram',...
            0.025:0.05:0.975,'Leverage score',...
            nanmean(leverageScoreHists,1),'Bin height');
return;

%% Leverage subsampling
f1Agreement = zeros(50,1);
compTimeLev = zeros(50,1);
for sr=1:50
    sampleRate = 0.02*sr;
    tic;
    linearityMatrixLev = zeros(yWindows,xWindows);
    for x=1:xWindows
        for y=1:yWindows
            edgeSample = BW((1+(y-1)*windowHeight):(y*windowHeight),(1+(x-1)*windowWidth):(x*windowWidth));
            [ yWhite, xWhite ] = find(edgeSample == 1);
            xWhiteOnes = [ ones(length(xWhite),1) xWhite ];
            yWhiteOnes = [ ones(length(yWhite),1) yWhite ];
            linearity = 10;
            if(length(xWhite) >= 5)
%                 if(ceil(sampleRate*length(xWhite)) > 5)
                    % Hat matrix leverage scores:
                    h = diag(xWhiteOnes*((xWhiteOnes'*xWhiteOnes) \ xWhiteOnes'));
                    if(sum(h) < 10e-6)
                        h = (1/length(xWhite))*ones(length(xWhite),1);
                    end
                    h = h / sum(h);
                    % Uniform leverage scores:
%                     h = (1/length(xWhite))*ones(length(xWhite),1);
                    sampleSize = min(5,ceil(sampleRate*length(xWhite)));
                    sample = nonuniform_sampling(h,sampleSize);
                    xWhiteSample = xWhiteOnes(sample,:);
                    yWhiteSample = yWhite(sample);
                    beta = ((xWhiteSample'*xWhiteSample) \ xWhiteSample')*yWhiteSample;
                    yHat = xWhiteOnes*beta;
                    residuals = abs(yWhite - yHat);
                    linearityX = nanmean(residuals);

                    h = diag(yWhiteOnes*((yWhiteOnes'*yWhiteOnes) \ yWhiteOnes'));
                    if(sum(h) < 10e-6)
                        h = (1/length(xWhite))*ones(length(xWhite),1);
                    end
                    h = h / sum(h);
                    sample = nonuniform_sampling(h,sampleSize);
                    yWhiteSample = yWhiteOnes(sample,:);
                    xWhiteSample = xWhite(sample);
                    beta = ((yWhiteSample'*yWhiteSample) \ yWhiteSample')*xWhiteSample;
                    xHat = yWhiteOnes*beta;
                    residuals = abs(xWhite - xHat);
                    linearityY = nanmean(residuals);
%                 else
%                     beta = ((xWhiteOnes'*xWhiteOnes) \ xWhiteOnes')*yWhite;
%                     yHat = xWhiteOnes*beta;
%                     residuals = abs(yWhite - yHat);
%                     linearityX = mean(residuals);
% 
%                     beta = ((yWhiteOnes'*yWhiteOnes) \ yWhiteOnes')*xWhite;
%                     xHat = yWhiteOnes*beta;
%                     residuals = abs(xWhite - xHat);
%                     linearityY = nanmean(residuals);
%                     linearity = nanmin(linearityX,linearityY);
%                 end
                linearity = nanmin(linearityX,linearityY);
            end
            linearityMatrixLev(y,x) = linearity;
        end
    end
    linearityMatrixLev = linearityMatrixLev / nanmax(linearityMatrixLev(:));
    thresholdedLinearityMatrixLev = linearityMatrixLev<0.4*nanmean(linearityMatrixLev(:));
    compTimeLev(sr) = toc;
    found = sum(thresholdedLinearityMatrixLev(:));
    
    recall = sum(thresholdedLinearityMatrixFull(:).*thresholdedLinearityMatrixLev(:)) / positives;
    precision = sum(thresholdedLinearityMatrixFull(:).*thresholdedLinearityMatrixLev(:)) / found;
    F1score = 2*precision*recall / (precision + recall);
    f1Agreement(sr) = F1score;
end
figure, imagesc(thresholdedLinearityMatrixLev), colormap gray;
figure, imagesc(linearityMatrixLev), colormap gray;

reportPlot('plot','F1 score of linearity detection versus sampling rate',...
            0.02:0.02:1,'Sampling rate',...
            f1Agreement,'F1 score');
reportPlot('plot','Linearity computation time versus sampling rate',...
            0.02:0.02:1,'Sampling rate',...
            compTimeLev,'Computation time');
reportPlot('plot','Computation time speedup versus sampling rate',...
            0.02:0.02:1,'Sampling rate',...
            (compTimeFull*ones(50,1))./compTimeLev,'Speedup');