function [ linearityBins, nonRandomness ] = linearity_detector( I )
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

if(size(I,3) > 1)
    Igray = imadjust(rgb2gray(I));
    scale = 600 / max(size(I));
else
    scale = 600 / max(size(I));
    Igray = imadjust(I);
end

Igray = imresize(Igray,scale);
BW = edge(Igray,'canny');
% imshow(BW);
% waitforbuttonpress;

width = size(BW,2);
height = size(BW,1);

windowWidth = min([10, size(Igray)]);
windowHeight = min([10, size(Igray)]);

xWindows = floor(width/windowWidth);
yWindows = floor(height/windowHeight);

linearityMatrix = zeros(yWindows,xWindows);
for x=1:xWindows
    for y=1:yWindows
        sample = BW((1+(y-1)*windowHeight):(y*windowHeight),(1+(x-1)*windowWidth):(x*windowWidth));
        [ yWhite, xWhite ] = find(sample == 1);
        linearity = NaN;
        if(length(xWhite) >= 5)
            p = polyfit(xWhite,yWhite,1);
            yFit = polyval(p,xWhite);
            residuals = abs(yWhite - yFit);
            linearityX = mean(residuals);
            
            p = polyfit(yWhite,xWhite,1);
            xFit = polyval(p,yWhite);
            residuals = abs(xWhite - xFit);
            linearityY = mean(residuals);
            linearity = min(linearityX,linearityY);
        end
        linearityMatrix(y,x) = linearity;
    end
end
linearityMatrix = linearityMatrix / max(linearityMatrix(:));

nonNaNlinearityMatrix = linearityMatrix(~isnan(linearityMatrix));
linearityBins = hist(nonNaNlinearityMatrix(:),10);
linearityBins = linearityBins / sum(linearityBins);

% Non randomness
sortedLinearityScores = sort(nonNaNlinearityMatrix(:));
ThresholdPercent = sortedLinearityScores(ceil(0.2*length(sortedLinearityScores)));
thresholdImage = ((linearityMatrix < ThresholdPercent) .* ~isnan(linearityMatrix));
[ xThresholdPoints, yThresholdPoints ] = find(thresholdImage > 0.5);
try
    tri = delaunay(xThresholdPoints, yThresholdPoints);
    %figure, trisurf(tri,xThresholdPoints,yThresholdPoints,zeros(length(xThresholdPoints),1));
    triangleArea = zeros(size(tri,1),1);
    for i=1:size(tri,1)
        triangleArea(i) = polyarea(xThresholdPoints(tri(i,:)),yThresholdPoints(tri(i,:)));
    end
    binCount = 10;
    %figure, hist(triangleArea,binCount);
    nonRandomnessBins = hist(triangleArea,binCount);
    nonRandomnessBins = nonRandomnessBins / sum(nonRandomnessBins);
    nonRandomness = nonRandomnessBins; %sum(abs(nonRandomnessBins - (1/binCount)));
catch
    nonRandomness = NaN(10,1);
end

end

