%% DESCRIPTION
% Scale video frames to a very low resolution (e.g 30x50). Each pixel is
% treated as a dimension. The distance between frames is calculated and
% searched for significant pattern.
% We then apply random projections and verify a correlation between the
% original distance and the low-dimension distance.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end

lowWidth = 40;
lowHeight = 30;
dimension = lowWidth*lowHeight;
%nFrames=960;

%% Load low-res frames and vectorize
disp('Build data set');
tic;
X = zeros(nFrames,dimension);
for frame=1:nFrames
    Ilowgray = imresize(small_mov(frame).cdata,[ lowHeight, lowWidth]);
    X(frame,:) = Ilowgray(:);
end
toc;

%% Calculate distance between frames
disp('Full distance matrix');
tic;
distanceVector = zeros(nFrames-1,1);
for frame=2:nFrames
   distanceVector(frame-1) =  sqrt(sum((X(frame-1,:)-X(frame,:)).^2));
end
toc;

%% Random projection
% Generate random matrix with normal distributed entries
disp('Random projection');
tic;
subDimension = ceil(10*log(dimension));
RP = mvnrnd(zeros(subDimension,dimension),eye(dimension));
rpX = X*RP';
rpDistanceVector = zeros(nFrames-1,1);
for frame=2:nFrames
   rpDistanceVector(frame-1) =  sqrt(sum((rpX(frame-1,:)-rpX(frame,:)).^2));
end
toc;

%% Detect scene shifts
trueCuts = [ 188, 208, 225, 261, 296, 333, 374, 387, 410, 427, 442, 479, 500, 567, 589, 613, 628, 649, 662, 673, 694, 698, 739, 756, 771, 805, 819, 832, 841, 854, 945, 1159, 1206, 1503, 1607, 2122, 2208, 2356, 2435, 2523, 2567, 2593, 2607, 2628, 2658, 2692, 2724, 2750, 2784, 2805, 2937 ];
% trueCuts = [ 16, 67, 137, 187, 244, 272, 347, 399, 416 ];
figure, plot(2:nFrames,rpDistanceVector);
cutThreshold = 0.3;
[ cutsFull, ~ ] = cut_detection_frame_distance(distanceVector,cutThreshold);
truePositives = sum(ismember(cutsFull,trueCuts));
precision = truePositives / length(cutsFull)
recall = truePositives / length(trueCuts)
[ cutsRP, cutsDetected ] = cut_detection_frame_distance(rpDistanceVector,cutThreshold);
truePositives = sum(ismember(cutsRP,trueCuts));
precision = truePositives / length(cutsRP)
recall = truePositives / length(trueCuts)

% figure();
% for cut=1:cutsDetected
%     frameIdx = cutsRP(cut);
%     imagesc(cat(2,small_mov(frameIdx-1).cdata,small_mov(frameIdx).cdata)), colormap(gray), axis image off;
%     title(strcat('Frames ',num2str(frameIdx-1),' + ',num2str(frameIdx)));
%     waitforbuttonpress;
% end
return;

%% Determine best thresholds
% For Zelotypia
trueCuts = [ 16, 67, 137, 187, 244, 272, 347, 399, 416 ];
F1score = zeros(20,1);
for i=1:20
    cutThreshold = 0.05*i;
    [ cuts, cutsDetected ] = cut_detection_frame_distance(distanceVector,cutThreshold);
    truePositives = sum(ismember(cuts,trueCuts));
    precision = truePositives / length(cuts);
    recall = truePositives / length(trueCuts);
    F1score(i) = 2*precision*recall / (precision + recall);
end
figure, plot(0.05:0.05:1,F1score,'r*-'); title('Adaptive threshold - Full distance');
F1score = zeros(20,1);
for i=1:20
    cutThreshold = 0.05*i;
    [ cuts, cutsDetected ] = cut_detection_frame_distance(rpDistanceVector,cutThreshold);
    truePositives = sum(ismember(cuts,trueCuts));
    precision = truePositives / length(cuts);
    recall = truePositives / length(trueCuts);
    F1score(i) = 2*precision*recall / (precision + recall);
end
figure, plot(0.05:0.05:1,F1score,'r*-'); title('Adaptive threshold - Random projection distance');