%% DESCRIPTION
% This script shows the mean camera or object movement based on the
% displacement of the SIFT features. A combination of two adjacent frames
% and the matched SIFT features is shown, as well as an historgram of the
% x- and y-displacement distribution and a plot with a 10 frame-history
% (trace) of the average displacement.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
run('/home/jan/dtu/speciale/matlab/vlfeat-0.9.18/toolbox/vl_setup');
addpath('..');
addpath('../leverageScores');
tic;
visualize = 0;
%% Cut detection settings
motionDirection = zeros(10,2);
displacement = zeros(nFrames,2);
displacement(1,:) = [ 0, 0 ];
matchCountMatrix = zeros(nFrames-1,1);
diffMatchCountMatrix = zeros(nFrames-2,1);
diffThreshold = zeros(nFrames-2,1);
cuts = [];
cutsDetected = 0;
histDiffMatrix = zeros(nFrames-1,1);

if(visualize)
    figure(1), set(gcf,'Units','normalized','Position',[ 0.07, 0.70, 0.3 0.3 ]);
    figure(2), set(gcf,'Units','normalized','Position',[ 0.68, 0.7, 0.3 0.3 ]);
    figure(3), set(gcf,'Units','normalized','Position',[ 0, 0, 1 1 ]);
end

%% Patch settings
patchWidth = floor(0.3*smallWidth);
patchHeight = floor(0.3*smallHeight);
nPatches = 20;
nSampledPatches = ceil(0.3*nPatches);
patchPositionX = floor((smallWidth-patchWidth)*rand(nPatches,1))+1;
patchPositionY = floor((smallHeight-patchHeight)*rand(nPatches,1))+1;

%% Cut detection
for i=1:(nFrames-1)
    if(mod(i,100) == 0)
        disp(i);
    end
    Ia = single(small_mov(i).cdata);
    Ib = single(small_mov(i+1).cdata);
    [fb,db] = vl_sift(Ib);
    
    % Extract patches
    patches = zeros(patchHeight,patchWidth,nPatches);
    variation = zeros(nPatches,1);
    for patch=1:nPatches
        patches(:,:,patch) = Ia(patchPositionY(patch):(patchPositionY(patch)+patchHeight-1),...
                        patchPositionX(patch):(patchPositionX(patch)+patchWidth-1));
        variation(patch) = std(reshape(patches(:,:,patch),patchWidth*patchHeight,1));
    end
    variation = variation / sum(variation);
    % Select good patches
    selectedPatches = nonuniform_sampling(variation,nSampledPatches);
    % Match matches to next frame and count matches
    matchCount = 0;
    for j=1:nSampledPatches
        [fa,da] = vl_sift(single(patches(:,:,selectedPatches(j))));
        [matches, scores] = vl_ubcmatch(da,db);
        matchCount = matchCount + size(matches,2);
    end

    % Determine how many pixels exceed the difference threshold
   matchCountMatrix(i) = matchCount;
   if(i > 1)
       diffMatchCountMatrix(i-1) = abs(matchCountMatrix(i-1) - matchCountMatrix(i));
   end
   if(i > 10 && (cutsDetected == 0 || i-cuts(cutsDetected) > 10))
       meanDiff = mean(diffMatchCountMatrix((i-10):(i-2)));
       varDiff = var(diffMatchCountMatrix((i-10):(i-2)));
       diffThreshold(i-1) = 2.5*meanDiff + 3*sqrt(varDiff);
   elseif(i > 1)
       diffThreshold(i-1) = 0.0010*smallWidth*smallHeight;
   end
   
%    histBins = 20;
%    histDifference = sum(abs(hist(Ia(:),histBins)-hist(Ib(:),histBins)));
%    histDiffMatrix(i) = histDifference;
   % Detect cut if the number of pixels above the difference threshold
   % exceeds the cut threshold
  if( i > 1 && diffMatchCountMatrix(i-1) > diffThreshold(i-1))%...
%           && histDifference > histBins*0.0125*smallHeight*smallWidth)
      if(cutsDetected == 0 || ...
         (cutsDetected > 0 && i-cuts(cutsDetected) > 1))
           cutsDetected = cutsDetected + 1;
           cuts(cutsDetected) = i + 1;
      end
   end
   
   [drop, perm] = sort(scores, 'descend') ;
    matches = matches(:, perm) ;
    scores  = scores(perm) ;
    
    % Find the inter-frame displacement
    motionDirection(1:9,:) = motionDirection(2:10,:);
    motionDirection(10,:) = median(fa(1:2,matches(1,:))'-fb(1:2,matches(2,:))');
    displacement(i+1,:) = displacement(i,:) + motionDirection(10,:);
    if((i > 1 && diffMatchCountMatrix(i-1) > 200) || displacement(i+1,1) > 300 || displacement(i+1,2) > 150)
        displacement(i+1,:) = [ 0, 0 ];
    end

    % Visualize the SIFT-matching
    if(visualize)
        figure(1); clf;
        hist(fa(1:2,matches(1,:))'-fb(1:2,matches(2,:))',-50:5:50);
        figure(2); clf;
        plot(motionDirection(:,1),motionDirection(:,2),'O-',0,0,'r*');
        axis([-70 70 -70 70 ]);


        figure(3) ; clf ;
        imagesc(cat(2, Ia, Ib)) ;

        xa = fa(1,matches(1,:)) ;
        xb = fb(1,matches(2,:)) + size(Ia,2) ;
        ya = fa(2,matches(1,:)) ;
        yb = fb(2,matches(2,:)) ;

        hold on ;
        h = line([xa ; xb], [ya ; yb]) ;
        set(h,'linewidth', 1, 'color', 'b') ;

        vl_plotframe(fa(:,matches(1,:))) ;
        fb(1,:) = fb(1,:) + size(Ia,2) ;
        vl_plotframe(fb(:,matches(2,:))) ;
        axis image off, colormap gray;
        % Matching visualization:
        vl_demo_print('sift_match_2', 1) ;

        % Stabilized image:
        figure(4); clf ;
        image(Ia), colormap gray, axis([ -displacement(i,1) 512-displacement(i,1) -displacement(i,2) 206-displacement(i,2) ]);
        waitforbuttonpress;
    end
    
end
toc;
t = 1:(nFrames-1);
reportPlot('plot','SIFT matches between adjacent frames',t,'Frame',matchCountMatrix,'# Matches');
t = 2:(nFrames-1);
reportPlot('plot','Difference of SIFT matches between adjacent frames',t,'Frame',[ diffMatchCountMatrix diffThreshold ],'Difference');

% figure();
% for cut=1:cutsDetected
%     frameIdx = cuts(cut);
%     imagesc(cat(2,small_mov(frameIdx-1).cdata,small_mov(frameIdx).cdata));
%     title(strcat('Frames ',num2str(frameIdx-1),' + ',num2str(frameIdx)));
%     waitforbuttonpress;
% end