%% DESCRIPTION
% Calculate the generalized leverage scores for linear regression, using
% the objective function and example-data (see thesis for details).

close all;
clear all;

run('../plot_settings.m');

n = 1000;
p = 2;


%% X-data definitions:
% #### Definition 1: #####
%offset = 5*rand(1,p);
%X = [ randn([ n/2, p ]) +  kron(ones(n/2,1),offset) ; 2*randn([ n/2, p ]) ];

% #### Definition 2: #####
%X = randn([ n, p ]);

% #### Definition GPA: #####
MU = ones(n,p);
SIGMA = zeros(p,p);
for i=1:p
    for j=1:p
        SIGMA(i,j) = 2^(-abs(i-j)+1);
    end
end
X = mvnrnd(MU,SIGMA);

% #### Definition T3: #####
%X = mvtrnd(SIGMA,3,n);

% #### Definition T1: #####
%X = mvtrnd(SIGMA,1,n);

% Centering:
for i=1:p
    X(:,i) = X(:,i) - mean(X(:,i));
end
Xmax = max(max(X(:,1)))*1.1;
Xmin = min(min(X(:,1)))*1.1;
Ymax = max(max(X(:,2)))*1.1;
Ymin = min(min(X(:,2)))*1.1;

% data-distance from center:
% Xdist = sum(X.^2,2);
% distFig = figure();
% hist(Xdist,50);

% Beta & noise
beta = ones(p,1);
if(p > 20)
    beta(11:p-10) = 0.1;
end
noise = normrnd(0,9,[ n, 1 ]);

%% Hat matrix:
H = X * ((X'*X) \ X');
h = diag(H);
h = h / sum(h);

hatHistFig = figure();
hist(h,0.00025:0.0005:0.00975);
title('hatMatrix');


%% Response vector with noise:
%y = X * beta + (meanH./h).*noise; % Only leverage-dependent noise can influence
                                  % difference between uniform and leverage-based sampling
                                  % for multivariate distribution.

y = X * beta + noise;       % This gives the desired graphs for the multivariate-
                            % t-distributions with low degree of freedom.


linearFit = LinearModel.fit(X,y);
LeverageScores = leverage(X);
LeverageScores = LeverageScores / sum(LeverageScores);
CookDScores = linearFit.Diagnostics.CooksDistance;
figure();
plot(LeverageScores,CookDScores,'O');
title('LeverageScores vs CookDScores');
figure();
plot(LeverageScores,h,'O');
title('LeverageScores vs hatMatrix');

                            
if(p==2)
    figure();
    scatter(X(:,1),X(:,2),plotScatterSize,h);
    title('hatMatrix');
    axis([Xmin Xmax Ymin Ymax]);
    caxis([ 0 0.005 ]);
    
    figure();
    scatter(X(:,1),X(:,2),plotScatterSize,LeverageScores);
    title('LeverageScores');
    axis([Xmin Xmax Ymin Ymax]);
    caxis([ 0 0.005 ]);
    
    figure();
    scatter(X(:,1),X(:,2),plotScatterSize,h);
    title('Cooks distance');
    axis([Xmin Xmax Ymin Ymax]);
end

%% Generalized leverage scores:

% Qda = zeros(p,1);
% 
% for i=1:p
%     sumV = 0;
%     for j=1:n
%         sumV = sumV + (y(j) - X(j,:)*beta)*X(j,i);
%     end
%     Qda(i) = -2*sumV;
% end
% 
% Qdada = zeros(p,p);
% 
% for i=1:p
%     for j=1:p
%         sumV = 0;
%         for k=1:n
%             sumV = sumV + beta(j)*X(k,i)*X(k,j);
%         end
%         Qdada(i,j) = 2*sumV;
%     end
% end
% 
% QdadY = zeros(p,n);
% 
% for i=1:p
%     for j=1:n
%         QdadY(i,j) = -2*y(j)*X(j,i);
%     end
% end
% 
% Da = zeros(n,p);
% 
% for i=1:n
%     for j=1:p
%         Da(i,j) = X(i,j)*beta(j);
%     end
% end

X2 = X;%x2fx(X);
beta2 = beta;%[ 0; beta ];
p = p-1;

Qdada = zeros(p+1,p+1);

for i=1:(p+1)
    for j=1:(p+1)
        sumV = 0;
        for k=1:n
            sumV = sumV + beta2(j)*X2(k,i)*X2(k,j);
        end
        Qdada(i,j) = 2*sumV;
    end
end

QdadY = zeros(p+1,n);

for i=1:(p+1)
    for j=1:n
        QdadY(i,j) = -2*X2(j,i);
    end
end

% d mu / da
Da = zeros(n,p+1);

for i=1:n
    for j=1:(p+1)
        Da(i,j) = X2(i,j)*beta2(j);
    end
end


GL = Da * (-Qdada \ QdadY);
GLdiag = abs(diag(GL));
GLdiag = GLdiag / sum(GLdiag);

GLhistFig = figure();
hist(GLdiag,0.00025:0.0005:0.00975);

if(p==2)
    figure();
    scatter(X(:,1),X(:,2),plotScatterSize,GLdiag);
    axis([Xmin Xmax Ymin Ymax]);
    caxis([ 0 0.01 ]);
end

hatGLcompareFig = figure();
plot(h,GLdiag,'O');

tri = delaunay(X(:,1),X(:,2));
figure();
trisurf(tri,X(:,1),X(:,2),GLdiag);
figure();
trisurf(tri,X(:,1),X(:,2),h);