%% DESCRIPTION
% This script uses stocastic simulation to find the generalized leverage
% scores for SVM and a given data set. The method is to run several sampled
% SVM models for each observation and determining if the prediction of this
% observation is changed if its label is changed.
% CAUTION: Takes a long time to finish even for small data sets.

close all;
clear all;

run('../plot_settings.m');

% Set data parameters
n = 500;
p = 2;
noiseLevel = 3;

% Generate data and visualize them
[ X, Y ] = generate_data(n,p,noiseLevel,'normal');
Y = (Y > median(Y)) + 1;
figure, scatter(X(:,1),X(:,2),20,Y,'LineWidth',2);

% Set algorithm parameters (see thesis for details)
M = 100;        % Subset size
k = 10;         % Simulations per observation

tic;
derivative = zeros(n,k);
parfor j=1:n
    for c1=1:k
        % Draw subset of size M
        sample = ceil(n*rand(M,1));
        Xsample = [X(sample,:) ; X(j,:)];
        Ysample = [Y(sample) ; Y(j) ];
        
        % Train model
        SVMModel = svmtrain(Xsample,Ysample);
        value1 = svmclassify(SVMModel,Xsample(M+1,:));
        
        % Modify the response vector Y
        if(Ysample(M+1) == 1)
            Ysample(M+1) = 2;
        else
            Ysample(M+1) = 1;
        end
        
        % Re-train the model
        SVMModel = svmtrain(Xsample,Ysample);
        value2 = svmclassify(SVMModel,Xsample(M+1,:));
        
        % Store the derivative
        derivative(j,c1) = value2 - value1;
        
    end
end
toc;

% Visualize the results
derivativeMean = mean(derivative,2);
derivativeVar = var(derivative,0,2);

figure, scatter(X(:,1),X(:,2),20,abs(derivativeMean));
tri = delaunay(X(:,1),X(:,2));
figure,trisurf(tri,X(:,1),X(:,2),abs(derivativeMean));

[Xgrid,Ygrid] = meshgrid(-6:0.1:6,-6:0.1:6);
Zgrid = griddata(X(:,1),X(:,2),abs(derivativeMean),Xgrid,Ygrid);
figure, surface(Xgrid,Ygrid,Zgrid,'LineStyle','none','FaceAlpha',0.7);
hold on; scatter(X(:,1),X(:,2),30,0.2*(Y-1),'*','LineWidth',1);hold off;