function [ X, y, beta ] = generate_data( n, p, noiseLevel, distribution, df )
%GENERATE_DATA Generates data sets with a multivariate normal- or
%t-distribution.
%   Valid input for distribution is
%   - normal
%   - t
%   - skewed (designed to be assymmetric)
%   When distribution is 't', df (Degree of Freedom) has to be specified

%% X-data definitions:
% #### Definition 1: #####
%offset = 5*rand(1,p);
%X = [ randn([ n/2, p ]) +  kron(ones(n/2,1),offset) ; 2*randn([ n/2, p ]) ];

% #### Definition 2: #####
%X = randn([ n, p ]);

% #### Definition GPA: #####
MU = ones(n,p);
SIGMA = zeros(p,p);
for i=1:p
    for j=1:p
        SIGMA(i,j) = 2^(-abs(i-j)+1);
    end
end

if(strcmp(distribution,'t'))
    % #### Definition TX (X=degree of freedom): #####
    X = mvtrnd(SIGMA,df,n);
elseif(strcmp(distribution,'skewed'))
    X = mvnrnd(MU,SIGMA);
    X(5:5:end,:) = X(5:5:end,:) + 5;
else
    % #### Definition GA: #####
    X = mvnrnd(MU,SIGMA);
end

% Centering:
for i=1:p
    X(:,i) = X(:,i) - mean(X(:,i));
end

% Beta & noise
beta = ones(p,1);
if(p > 20)
    beta(11:p-10) = 0.1;
end
noise = normrnd(0,noiseLevel,[ n, 1 ]);

%% Response vector with noise:
y = X * beta + noise;       % This gives the desired graphs for the multivariate-
                            % t-distributions with low degree of freedom.



end

