%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by linear_mean_var_experiments_script.m
% Perform linear regression experiment with sampled data, sampled according
% to the alternative K-nearest neighbors weighting (see thesis and below).
% We use reweighting to obtain unbiased matrix products

if(showScatterPlots == 1)
    figure(scatterFig);
end

%% Calculate the importance measure
K = 10; % Number of neighbors considered

% We can use the median or mean distance to the K-nearest neighbors as
% importance measure. Uncomment the appropriate lines:

% medianKneighborDistance = zeros(n,1);
meanKneighborDistance = zeros(n,1);
% Find median or mean distance to the K-nearest neighbors for each
% observation
for i=1:n
    distance = sqrt(sum((X-repmat(X(i,:),n,1)).^2,2));
    idxDistanceMatrix = [ (1:n)' , distance ];
    sortedIdxDistanceMatrix = sortrows(idxDistanceMatrix,2);
    % Find median and mean. We use 2:11 to discard the distance to the
    % observation itself, which is always 0.
%     medianKneighborDistance(i) = median(sortedIdxDistanceMatrix(2:(1+K),2));
    meanKneighborDistance(i) = mean(sortedIdxDistanceMatrix(2:(1+K),2));
end
% h = medianKneighborDistance / sum(medianKneighborDistance);
h = meanKneighborDistance / sum(meanKneighborDistance);


% Set reference-\beta
betaOLS = ((X'*X) \ X')*y;
if(conditional)
    beta = betaOLS;
else
    beta = beta0;
end

% Prepare result variables:
betaHat = zeros(p,experiments,repetitions);
meanBetaKNN = zeros(p,experiments);
varBetaKNN = zeros(experiments,1);
biasKNN = zeros(experiments,1);
biasConfidenceIntervalKNN = zeros(experiments,1);
varBetaConfidenceIntervalKNN = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    experimentError = zeros(repetitions,1);
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            noise = normrnd(0,noiseLevel,[ n, 1 ]);
            y = X*beta + noise;
        end
        sampleSize = stepSize*experiment;
        sample = nonuniform_sampling(h,sampleSize);
        % Calculate reweighting-matrix
        D = zeros(sampleSize);
        for i=1:sampleSize
            D(i,i) = sqrt(1/(sampleSize*h(sample(i))));
        end

        % Build subsampled data set and reweight
        Xsample = D*X(sample,:);
        Ysample = D*y(sample);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Leverage - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        
        % Save \beta
        betaHat(:,experiment,rep) = ((Xsample'*Xsample) \ Xsample') * Ysample;
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaKNN(i,experiment) = mean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasKNN(experiment) = log(sum((mean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaKNN(experiment) = log(mean(var(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((mean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalKNN(experiment) = std(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(mean(var(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalKNN(experiment) = std(bootstrapVar);
end

%% Visualize results
% Bias:
figure(errorFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,biasKNN,biasConfidenceIntervalKNN(:),'k','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (reweighted)','Leverage','KNN importance','Location','Best');

% Average \beta values
figure(meanBetaFig);
hold on
plot(stepSize:stepSize:stepSize*experiments,mean(meanBetaKNN,1),'k','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (reweighted)','Leverage','KNN importance','Location','Best');

% Variance
figure(varBetaFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,varBetaKNN,varBetaConfidenceIntervalKNN(:),'k','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (reweighted)','Leverage','KNN importance','Location','Best');
