%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by linear_mean_var_experiments_script.m
% Perform linear regression experiment with sampled data, sampled according
% to the hat matrix leverage scores (without reweighting).

if(showScatterPlots == 1)
    figure(scatterFig);
end

% Calculate the importance measure
H = X*((X'*X) \ X');
h = diag(H);
h = h / sum(h);

% Set reference-\beta
if(~conditional)
    beta = beta0;
end

% Prepare result variables
betaHat = zeros(p,experiments,repetitions);
meanBetaLeverage = zeros(p,experiments);
varBetaLeverage = zeros(experiments,1);
biasLeverage = zeros(experiments,1);
biasConfidenceIntervalLeverage = zeros(experiments,1);
varBetaConfidenceIntervalLeverage = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            noise = normrnd(0,noiseLevel,[ n, 1 ]);
            y = X*beta + noise;
        end
        sampleSize = stepSize*experiment;
        sample = nonuniform_sampling(h,sampleSize);

        % Build subsampled data set and reweight
        Xsample = X(sample,:);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Leverage - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        
        % Set reference-\beta
        if(conditional)
            W0 = diag(sampleSize*h);
            betaWLS = ((X'*W0*X) \ X'*W0)*y;
            beta = betaWLS;
        end
        % Save \beta
        betaHat(:,experiment,rep) = ((Xsample'*Xsample) \ Xsample') * y(sample);
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaLeverage(i,experiment) = mean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasLeverage(experiment) = log(sum((mean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaLeverage(experiment) = log(mean(var(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((mean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalLeverage(experiment) = std(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(mean(var(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalLeverage(experiment) = std(bootstrapVar);
end

%% Visualize results
% Bias:
figure(errorFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,biasLeverage,biasConfidenceIntervalLeverage(:,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage (Reweighted)','Leverage','Location','Best');

% Average \beta values
figure(meanBetaFig);
hold on
plot(stepSize:stepSize:stepSize*experiments,mean(meanBetaLeverage,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage (Reweighted)','Leverage','Location','Best');

% Variance
figure(varBetaFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,varBetaLeverage,varBetaConfidenceIntervalLeverage(:,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage (Reweighted)','Leverage','Location','Best');
