%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by linear_mean_var_experiments_script.m
% Perform linear regression experiment with sampled data, sampled according
% to the hat matrix leverage scores. Reweight observations in order to
% obtain unbiased matrix products.

if(showScatterPlots == 1)
    figure(scatterFig);
end

% Calculate the importance measure
H = X*((X'*X) \ X');
h = diag(H);
h = h / sum(h);

% Set reference-\beta
betaOLS = ((X'*X) \ X')*y;
if(conditional)
    beta = betaOLS;
else
    beta = beta0;
end

% Prepare result variables:
betaHat = zeros(p,experiments,repetitions);
meanBetaLeverageRescaled = zeros(p,experiments);
varBetaLeverageRescaled = zeros(experiments,1);
biasLeverageRescaled = zeros(experiments,1);
biasConfidenceIntervalLeverageRescaled = zeros(experiments,1);
varBetaConfidenceIntervalLeverageRescaled = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    experimentError = zeros(repetitions,1);
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            noise = normrnd(0,noiseLevel,[ n, 1 ]);
            y = X*beta + noise;
        end
        sampleSize = stepSize*experiment;
        sample = nonuniform_sampling(h,sampleSize);
        % Calculate reweighting-matrix
        D = zeros(sampleSize);
        for i=1:sampleSize
            D(i,i) = sqrt(1/(sampleSize*h(sample(i))));
        end

        % Build subsampled data set and reweight
        Xsample = D*X(sample,:);
        Ysample = D*y(sample);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Leverage - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        % Save \beta
        betaHat(:,experiment,rep) = ((Xsample'*Xsample) \ Xsample') * Ysample;
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaLeverageRescaled(i,experiment) = mean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasLeverageRescaled(experiment) = log(sum((mean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaLeverageRescaled(experiment) = log(mean(var(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((mean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalLeverageRescaled(experiment) = std(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(mean(var(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalLeverageRescaled(experiment) = std(bootstrapVar);
end

%% Visualize results
% Bias:
figure(errorFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,biasLeverageRescaled,biasConfidenceIntervalLeverageRescaled(:,1),'r','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (Reweighted)','Location','NorthEast');

% Average \beta values
figure(meanBetaFig);
hold on
plot(stepSize:stepSize:stepSize*experiments,mean(meanBetaLeverageRescaled,1),'r','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (Reweighted)','Location','NorthEast');

% Variance
figure(varBetaFig);
hold on
errorbar(stepSize:stepSize:stepSize*experiments,varBetaLeverageRescaled,varBetaConfidenceIntervalLeverageRescaled(:,1),'r','LineWidth',plotLineWidth);
hold off
legend('Uniform','Leverage (Reweighted)','Location','NorthEast');
