%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by linear_mean_var_experiments_script.m
% Perform linear regression experiment with sampled data, sampled according
% to a uniform random weight distribution.

if(showScatterPlots == 1)
    scatterFig = figure();
    pos = get(gcf, 'Position');
    set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
    set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
end

% Set reference-\beta
betaOLS = ((X'*X) \ X')*y;
if(conditional)
    beta = betaOLS;
else
    beta = beta0;
end

% Prepare result variables:
betaHat = zeros(p,experiments,repetitions);
meanBetaUniform = zeros(p,experiments);
varBetaUniform = zeros(experiments,1);
biasUniform = zeros(experiments,1);
biasConfidenceIntervalUniform = zeros(experiments,1);
varBetaConfidenceIntervalUniform = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            noise = normrnd(0,noiseLevel,[ n, 1 ]);
            y = X*beta0 + noise;
        end
        sampleSize = stepSize*experiment;
        sample = ceil(n*rand(sampleSize,1));
        % Calculate reweighting-matrix
        D = zeros(sampleSize);
        for i=1:sampleSize
            D(i,i) = sqrt(n/sampleSize);
        end

        % Build subsampled data set and reweight
        Xsample = D*X(sample,:);
        Ysample = D*y(sample);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Uniform - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        % Save \beta
        betaHat(:,experiment,rep) = ((Xsample'*Xsample) \ Xsample') * Ysample;
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaUniform(i,experiment) = mean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasUniform(experiment) = log(sum((mean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaUniform(experiment) = log(mean(var(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((mean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalUniform(experiment) = std(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(mean(var(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalUniform(experiment) = std(bootstrapVar);

end

%% Visualize results
% Bias:
errorFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
errorbar(stepSize:stepSize:stepSize*experiments,biasUniform,biasConfidenceIntervalUniform(:,1),'b--','LineWidth',plotLineWidth);
xlim([ 0, stepSize*experiments ]);
xlabel('Sample size');
ylabel('log(squared bias)');
if(conditional)
    title('Squared bias relative to \beta_{OLS}');
else
    title('Squared bias relative to \beta_{0}');
end
set(gca,'LineWidth',plotLineWidth);

% Average \beta values
meanBetaFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
plot(stepSize:stepSize:stepSize*experiments,mean(meanBetaUniform,1),'b--','LineWidth',plotLineWidth);
title('Average value of \beta elements');
set(gca,'LineWidth',plotLineWidth);

% Variance
varBetaFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
errorbar(stepSize:stepSize:stepSize*experiments,varBetaUniform,varBetaConfidenceIntervalUniform(:,1),'b--','LineWidth',plotLineWidth);
xlim([ 0, stepSize*experiments ]);
xlabel('Sample size');
ylabel('log(variance)');
title('Variance of \beta');
set(gca,'LineWidth',plotLineWidth);