%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by logistic_mean_var_experiments_script.m
% Perform logistic regression experiment with sampled data, sampled according
% to the hat matrix leverage scores. The data is not reweighted.

if(showScatterPlots == 1)
    figure(scatterFig);
end

% Set reference-\beta
if(conditional)
    beta = betaOLS;
else
    beta = beta0;
end

% Prepare result variables:
betaHat = zeros(p+1,experiments,repetitions);
meanBetaLeverage = zeros(p+1,experiments);
varBetaLeverage = zeros(experiments,1);
biasLeverage = zeros(experiments,1);
biasConfidenceIntervalLeverage = zeros(experiments,1);
varBetaConfidenceIntervalLeverage = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            y = (rand(n,1) > p0) + 1;
        end
        sampleSize = sampleSizeVector(experiment);
        sample = nonuniform_sampling(h,sampleSize);
        
        % Build subsampled data set and reweight
        Xsample = X(sample,:);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Leverage - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        
        % Save \beta
        try
            betaHat(:,experiment,rep) = mnrfit(Xsample,y(sample));
            yHat = [ ones(n,1) X ] * betaHat(:,experiment,rep);
            pHat = exp(yHat) ./ (1+exp(yHat));
            pHat(isnan(pHat)) = 1;
            pSum = nansum(abs(pHat-p0))
        catch
            betaHat(:,experiment,rep) = NaN;
        end
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaLeverage(i,experiment) = nanmean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasLeverage(experiment) = log(sum((nanmean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaLeverage(experiment) = log(nanmean(nanvar(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((nanmean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalLeverage(experiment) = nanstd(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(nanmean(nanvar(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalLeverage(experiment) = nanstd(bootstrapVar);
end

%% Visualize results
% Bias:
figure(errorFig);
hold on
errorbar(sampleSizeVector,biasLeverage,biasConfidenceIntervalLeverage(:,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage','Location','Best');

% Average \beta values
figure(meanBetaFig);
hold on
plot(sampleSizeVector,mean(meanBetaLeverage,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage','Location','Best');

% Variance
figure(varBetaFig);
hold on
errorbar(sampleSizeVector,varBetaLeverage,varBetaConfidenceIntervalLeverage(:,1),'-.','LineWidth',plotLineWidth,'Color',[0,0.5,0]);
hold off
legend('Uniform','Leverage','Location','Best');
