%% DESCRIPTION
% CANNOT RUN ON ITS OWN!
% This script is called by logistic_mean_var_experiments_script.m
% Perform logistic regression experiment with sampled data, sampled according
% to a uniform random weight distribution.

if(showScatterPlots == 1)
    scatterFig = figure();
    pos = get(gcf, 'Position');
    set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
    set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
end

% Set reference-\beta
if(conditional)
    beta = betaOLS;
else
    beta = beta0;
end

% Prepare result variables:
betaHat = zeros(p+1,experiments,repetitions);
meanBetaUniform = zeros(p+1,experiments);
varBetaUniform = zeros(experiments,1);
biasUniform = zeros(experiments,1);
biasConfidenceIntervalUniform = zeros(experiments,1);
varBetaConfidenceIntervalUniform = zeros(experiments,1);

%% Perform experiments
for experiment=1:experiments % Loop over sample sizes
    for rep=1:repetitions % Perform the experiments for each sample size several times
        if(~conditional)
            % Generate a new response for each experiment
            y = (rand(n,1) > p0) + 1;
        end
        sampleSize = sampleSizeVector(experiment);
        sample = ceil(n*rand(sampleSize,1));

        % Build subsampled data set and reweight
        Xsample = X(sample,:);
        % Show scatterplot of data, if desired
        if(experiment == floor(experiments/2) && showScatterPlots == 1)
            disp(strcat('Uniform - Experiment:',num2str(experiment),' Sample size:',num2str(sampleSize)));
            scatter(Xsample(:,1),Xsample(:,2),20,h(sample));
            axis([Xmin Xmax Ymin Ymax]);
            waitforbuttonpress;
        end
        
        % Save \beta
        try
            betaHat(:,experiment,rep) = mnrfit(Xsample,y(sample));
            yHat = [ ones(n,1) X ] * betaHat(:,experiment,rep);
            pHat = exp(yHat) ./ (1+exp(yHat));
            pHat(isnan(pHat)) = 1;
            pSum = nansum(abs(pHat-p0))
        catch
            betaHat(:,experiment,rep) = NaN;
        end
    end
    % Calculate average value of \beta
    for i=1:p
       meanBetaUniform(i,experiment) = nanmean(betaHat(i,experiment,:));
    end
    
    % Calculate bias and variance for the current sample size
    biasUniform(experiment) = log(sum((nanmean(betaHat(:,experiment,:),3) - beta).^2));
    varBetaUniform(experiment) = log(nanmean(nanvar(betaHat(:,experiment,:),0,3)));
    
    % Bootstrap std estimation for bias:
    bootstrapBias = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapBias(bs) = log(sum((nanmean(betaHat(:,experiment,bootstrapSamples),3) - beta).^2));
    end
    biasConfidenceIntervalUniform(experiment) = nanstd(bootstrapBias);
    
    % Bootstrap std estimation for variance:
    bootstrapVar = zeros(bootstrapRepetitions,1);
    for bs=1:bootstrapRepetitions
        bootstrapSamples = ceil(rep*rand(rep,1));
        bootstrapVar(bs) = log(mean(nanvar(betaHat(:,experiment,bootstrapSamples),0,3)));
    end
    varBetaConfidenceIntervalUniform(experiment) = nanstd(bootstrapVar);
end

%% Visualize results
% Bias:
errorFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
errorbar(sampleSizeVector,biasUniform,biasConfidenceIntervalUniform(:,1),'b--','LineWidth',plotLineWidth);
xlim([ 0, sampleSizeVector(experiments) ]);
xlabel('Sample size');
ylabel('log(squared bias)');
title('Error relative to true \beta');
set(gca,'LineWidth',plotLineWidth);

% Average \beta values
meanBetaFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
plot(sampleSizeVector,mean(meanBetaUniform,1),'b--','LineWidth',plotLineWidth);
title('Average value of \beta elements');
set(gca,'LineWidth',plotLineWidth);

% Variance
varBetaFig = figure();
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'FontSize', plotFontSize, 'LineWidth', plotLineWidth);
errorbar(sampleSizeVector,varBetaUniform,varBetaConfidenceIntervalUniform(:,1),'b--','LineWidth',plotLineWidth);
xlim([ 0, sampleSizeVector(experiments) ]);
xlabel('Sample size');
ylabel('log(variance)');
title('Variance of \beta');
set(gca,'LineWidth',plotLineWidth);
