function [ samples ] = nonuniform_sampling( probabilities, sampleSize )
%NONUNIFORM SAMPLING Returns index for datapoints in the range 1 to
%length(probabilities) according to the weights in probabilities
%   Detailed explanation goes here

samples = zeros(sampleSize,1);

if(abs(sum(probabilities) - 1) > 10e-6)
    disp('1st argument is not a probability distribution');
    return;
end

% Build commulative probability vector
collectionSize = length(probabilities);
cdf = zeros(collectionSize,1);
cdf(1) = probabilities(1);
for i=2:collectionSize
   cdf(i) = cdf(i-1) + probabilities(i); 
end

% The cdf divides the interval [0,1] into pieces of size proportional to
% the weights of the observations. Selecting a random number in this
% interval can be translated back to an observation. An observation with
% high weight is more likely to be 'hit'.

% Uniform sample in [0,1]:
p = rand(sampleSize,1);

% Use binary search:
searchSteps = ceil(log2(collectionSize));
for i=1:sampleSize
    collectionIndex = ceil(collectionSize/2);
    for s=2:searchSteps
        if(cdf(collectionIndex) < p(i))
           collectionIndex = collectionIndex + ceil(collectionSize/2^s);
        else
           collectionIndex = collectionIndex - ceil(collectionSize/2^s);
        end
        if(collectionIndex < 1)
            collectionIndex = 1;
        end
        if(collectionIndex > collectionSize)
            collectionIndex = collectionSize;
        end
    end
    if(cdf(collectionIndex) < p(i))
        collectionIndex = collectionIndex + 1;
    end
    samples(i) = collectionIndex;
end
end

