%% DESCRIPTION
% Experiments with approximation of matrix products by sampled rows/columns
% from the two matrices (see lemma 3 in "FAST MONTE CARLO ALGORITHMS FOR
% MATRICES I" by Drineras, Kannas & Mahoney
% Pay special attention to cases where the matrix product is
% biased/unbiased.
% Question: When can we discard the reweighting and still obtain an
% unbiased matrix product?

close all;
clear all;

% Matrix dimensions: A is a mxn-matrix, B is a nxp-matrix.
m = 300;
n = 800;
p = 300;
c = 50; % Number of rows/columns sampled (c < n)

[ A, ~ ] = generate_data(m,n,9,'t',3);
[ B, ~ ] = generate_data(n,p,9,'t',3);
% A = A - ones(m,1)*(rand(1,n)*n);
% B = B - ones(n,1)*(rand(1,p)*p);
%B = B + ones(m,n);
% B = A';
A = B';
C = zeros(m,c);
R = zeros(c,p);

H = B*((B'*B) \ B');
h = diag(H) / m;
% h = rand(n,1);
% h = (ones(n,1)+0.1*randn(n,1)) / n;
% h = exp(-0.03*(1:n));
h = h / sum(h);

%figure, hist(h,20);

%% Data exploration
% Find the bias E[CR] by simulation (see lemma 3 in "FAST MONTE CARLO
% ALGORITHMS FOR MATRICES I" by Drineras, Kannas & Mahoney
ABbias = zeros(m,p);
for i=1:m
    for j=1:p
        for k=1:n
%             ABbias(i,j) = ABbias(i,j) + A(i,k)*B(k,j)*h(k); % Weighted product approximation
            ABbias(i,j) = ABbias(i,j) + A(i,k)*B(k,j)/c; % Unweighted product approximation
        end
    end
end
diffABbias = c*ABbias-A*B;
% figure(1), imagesc(diffABbias), title('ABBias');
% figure(2), hist(diffABbias(:),20), title('ABBias');
% return;

%% Main script
%% Layer-approach
experiments = 100;
CRhistory = zeros(m,p,experiments);
AB = A*B;
ABLayers = zeros(m,p,n);
for i=1:n
   ABLayers(:,:,i) = A(:,i)*B(i,:);% / (c*h(i)); 
end
diffElementHistory = zeros(experiments,m*p);
for experiment=1:experiments
    sumABLayers = zeros(m,p);
    sample = nonuniform_sampling(h,c);
    for i=1:c
        sumABLayers = sumABLayers + ABLayers(:,:,sample(i));
    end
    CRhistory(:,:,experiment) = sumABLayers;
    diffABLayers = sumABLayers - AB;
    diffElementHistory(experiment,:) = diffABLayers(:);
end
figure, hist(diffElementHistory(:),20), title('Value distribution element (17,10)');
% figure, boxplot(diffElementHistory(:));
% mean(diffElementHistory(:))
ratioMatrix = mean(CRhistory,3) ./ AB;
ratioMatrix = ratioMatrix(ratioMatrix <= 1);
ratioMatrix = ratioMatrix(ratioMatrix > 0);
figure, boxplot(ratioMatrix(:));
return;

%% Original code column-row-wise

for experiment=1:experiments
    sample = nonuniform_sampling(h,c);
    for i=1:c
       C(:,i) = A(:,sample(i)) / sqrt(c*h(sample(i))); 
       R(i,:) = B(sample(i),:) / sqrt(c*h(sample(i)));
    end

    CR = zeros(m,p);
    CRlayerHistory = zeros(m,p,c);
    for i=1:c
        CR = CR + C(:,i)*R(i,:);
        CRlayerHistory(:,:,i) = C(:,i)*R(i,:);
    end
    CRlayerMean = mean(CRlayerHistory,3);
    diffCRLayer = CRlayerMean-AB;
    figure(3), hist(diffCRLayer(:),20);
    waitforbuttonpress;
%     diff = AB - CR;
%     figure, hist(diff(:),20);
    CRhistory(:,:,experiment) = CR;
end
CRmean = mean(CRhistory,3);
diff = AB - CRmean;
corr(AB(:),CRmean(:))
figure(4), hist(diff(:),20);


