%% DESCRIPTION
% Man-made object detection algorithm described in
% @article{cai2011man,
%   title={Man-made Object Detection Based on Texture Clustering and Geometric Structure Feature Extracting},
%   author={Cai, Fei and Chen, Honghui and Ma, Jianwei},
%   journal={International Journal of Information Technology and Computer Science (IJITCS)},
%   year={2011}
% }

close all;
clear all;

imagePath = '../../images/12_small.jpg';

I = imread(imagePath);
Igray = rgb2gray(imresize(I,0.5));

%% Preprocessing
% Wiener filter
[Iwiener,noise] = wiener2(Igray,[ 3, 3 ]);

% Median filter
Imedian = medfilt2(Iwiener);

% DCT transform
% J = dct2(Imedian);
% J(abs(J) < 10) = 0;
% Idct = idct2(J);

% Contrast enhancement
Icontrast = imadjust(Imedian);


%% Texture feature extraction
Ireducedgray = floor(Icontrast / 8);
levels = 33;
stdDeviationMoment = sqrt(statistical_moment(Ireducedgray,levels,2));
stdDeviationMatlab = std(single(Ireducedgray(:)));

% Tamura feature
alpha4 = statistical_moment(Ireducedgray,levels,4) / stdDeviationMoment^4;
Fcontrastness = stdDeviationMoment / nthroot(alpha4,4);

windowsX = 15;
windowsY = 20;
Imanmade = zeros(windowsY,windowsX);
Ialpha = zeros(windowsY,windowsX);
windowWidth = floor(size(Icontrast,2)/windowsX);
windowHeight = floor(size(Icontrast,1)/windowsY);
for x=1:windowsX
    for y=1:windowsY
        window = Icontrast(((y-1)*windowHeight+1):(y*windowHeight),...
                            ((x-1)*windowWidth+1):(x*windowWidth));
        windowReduced = floor(window / 8);
        stdDeviationMoment = sqrt(statistical_moment(windowReduced,levels,2));
        if(stdDeviationMoment == 0)
%             disp('std is 0');
            stdDeviationMoment = 1;
        end
        % Tamura feature
        alpha4 = statistical_moment(windowReduced,levels,4) / stdDeviationMoment^4;
        if(isnan(alpha4) || alpha4 == 0)
            alpha4 = 1;
        end
        Fcontrastness = stdDeviationMoment / nthroot(alpha4,4);
        
        Imanmade(y,x) = Fcontrastness;
        Ialpha(y,x) = alpha4;
    end
end

%% Visualize
imshow(Imanmade);
ImanmadeLarge = imresize(Imanmade,size(Icontrast));
waitforbuttonpress;
Ioverlay = double(Icontrast).*ImanmadeLarge;
image(Ioverlay);

