%% DESCRIPTION
% This script extracts SIFT-features from adjacent frames and usses the
% displacement of matched SIFT-features for detecting objects by using
% clustering-techniques.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
run('/home/jan/dtu/speciale/matlab/vlfeat-0.9.18/toolbox/vl_setup');
run('../plot_settings.m');
visualize = 1;

meanHistory = zeros(nFrames-2,1);
for i=1:(nFrames-2)
    Ia = single(small_mov(i).cdata);
    Ib = single(small_mov(i+1).cdata);
    Ic = single(small_mov(i+2).cdata);
    [fa,da] = vl_sift(Ia);
    [fb,db] = vl_sift(Ib);
   
   [matches, scores] = vl_ubcmatch(da,db,1.8);
    
   xDisplacement = fa(1,matches(1,:)) - fb(1,matches(2,:));
   yDisplacement = fa(2,matches(1,:)) - fb(2,matches(2,:));
   displacement = [ xDisplacement' yDisplacement' ];
   displacement(displacement > 10) = NaN;
   if(sum(~isnan(sum(displacement,2))) < 3)
       continue;
   end
   % K-Means clustering:
   clusterIDX = kmeans(displacement,2,'emptyaction','singleton');
   cluster1IDX = find(clusterIDX == 1);
   cluster2IDX = find(clusterIDX == 2);
   
%    Gaussian mixture model:
%    try
%        GMModel = gmdistribution.fit(displacement,2);
%        clusterIDX = cluster(GMModel,displacement);
%        cluster1IDX = find(clusterIDX == 1);
%        cluster2IDX = find(clusterIDX == 2);
%    catch
%        cluster1IDX = [];
%        cluster2IDX = [];
%    end
   
   if(length(clusterIDX) == size(displacement,1))
       SIFTpositions = [ fa(1,matches(1,:))', fa(2,matches(1,:))' ];
       SVMModel = svmtrain(SIFTpositions,clusterIDX,'kernel_function','rbf','rbf_sigma',0.5);
       clusterIDX = svmclassify(SVMModel,SIFTpositions);
       cluster1IDX = find(clusterIDX == 1);
       cluster2IDX = find(clusterIDX == 2);
   end
   if(length(cluster1IDX) < length(cluster2IDX))
       temp = cluster1IDX;
       cluster1IDX = cluster2IDX;
       cluster2IDX = temp;
   end
   cluster1mean = nanmean(displacement(cluster1IDX,:));
   cluster2mean = nanmean(displacement(cluster2IDX,:));
   meanDistance = sqrt(sum((cluster1mean-cluster2mean).^2));
   meanHistory(i) = meanDistance;
%    disp(meanDistance);
   cluster1size = length(cluster1IDX);
   cluster2size = length(cluster2IDX);
   clusterSizeRatio = cluster1size / cluster2size;
   disp(clusterSizeRatio);
   
    %% Visualization
    if(visualize)

        figure(1) ; clf ;
        imagesc(cat(2, Ia, Ib)) ;
        axis image off, colormap gray;

        if(length(cluster1IDX) + length(cluster2IDX) <= size(matches,2))
            xa = fa(1,matches(1,:)) ;
            xb = fb(1,matches(2,:)) + size(Ia,2) ;
            ya = fa(2,matches(1,:)) ;
            yb = fb(2,matches(2,:)) ;

            fb(1,:) = fb(1,:) + size(Ia,2) ;

            hold on ;
            % Plot lines between matches
            %h = line([xa ; xb], [ya ; yb]) ;
            %set(h,'linewidth', 1, 'color', 'b') ;

            % Plot SIFT-detectors
            % Cluster 1:
            fa(3,matches(1,:)) = scores*30/max(scores);
            vl_plotframe(fa(:,matches(1,cluster1IDX))) ;
            vl_plotframe(fb(:,matches(2,cluster1IDX))) ;
            % Cluster 2:
            vl_plotframe(fa(:,matches(1,cluster2IDX)),'Color','red') ;
            vl_plotframe(fb(:,matches(2,cluster2IDX)),'Color','red') ;
            hold off;
            title('Matched SIFT-features');
            % Matching visualization:
            %vl_demo_print('sift_match_2', 1) ;

            figure(2); clf;

            scatter(xDisplacement(cluster1IDX),yDisplacement(cluster1IDX),30,'blue','LineWidth',2);
            hold on;
            scatter(xDisplacement(cluster2IDX),yDisplacement(cluster2IDX),30,'red','LineWidth',2);
            hold off;
            axis([-15, 15, -15, 15]);
            xlabel('X'); ylabel('Y');
            title('SIFT-feature displacement')
        end
        waitforbuttonpress;
    end
    
end
