function [ Ireconst ] = pca_reconstruct( I, blockSize, pct )
%PCA_RECONSTRUCT Constructs an approximation of input image I using a given
%percentage of the data.
%   Detailed explanation goes here

bX = blockSize; % Blocksize in X-direction 
bY = blockSize; % Blocksize in Y-direction


%transform it to grayscale - use the commented lines if you have the image
%toolbox

% grayscale
if(size(I,3) > 1)
    Igray=rgb2gray(I);
else
    Igray = I;
end

% Crop the image to a size that it's side length are a multiple of the
% block size.
%I=I(1:1100,:);
Iwidth = size(Igray,2)-mod(size(Igray,2),bX);
Iheight = size(Igray,1)-mod(size(Igray,1),bY);
Igray=Igray(1:Iheight,1:Iwidth);
Igray = double(Igray);


%transform image to a measurement x channel matrix
%DataMatrix is a collection of 10x10 pixel image patches
Indx=0;
rowsOfBlocks = size(Igray,1)/bX;
columnsOfBlocks = size(Igray,2)/bY;
DataMatrix=zeros(ceil(rowsOfBlocks*columnsOfBlocks),bX*bY);

for r=1:floor(rowsOfBlocks)
    for c=1:floor(columnsOfBlocks)
        Indx=Indx+1;    
        DataMatrix(Indx,:)=reshape(I((r-1)*bX+(1:bX),(c-1)*bY+(1:bY)),[1 bX*bY]);
    end
end

% principal component analysis (PCA)
% X = coeff
% W = score
[X,W,~] = pca(DataMatrix);

componentsNeeded = min(max(1,round(size(X,2)*pct)),size(X,2));
    
% reconstruction by first few principal components
%S_reconstr = X*W'; % Full reconstruction
S_reconstr = [X(:,1:componentsNeeded) zeros(size(X,1),size(X,2)-componentsNeeded)]*W';
Ireconst = zeros(size(I,1),size(I,2));
Indx = 0;
for r=1:floor(rowsOfBlocks)
    for c=1:floor(columnsOfBlocks)   
        Indx=Indx+1;  
        Ireconst((r-1)*bX+(1:bX),(c-1)*bY+(1:bY)) = reshape(S_reconstr(:,Indx),bX,bY);
    end
end

Imin = min(Igray(:));
Imax = max(Igray(:));

Ireconst = Ireconst - (min(Ireconst(:))-Imin);
Ireconst = Ireconst * (Imax / max(Ireconst(:)));
Ireconst = round(Ireconst);

end

