function [ characteristics ] = camera_motion_characteristics( mov, cuts )
%CAMERA_MOTION_CHARACTERISTICS Give a characterization of the camera motion
%   Input:  - mov, a movie struct
%   Output: - characteristics, containing integers indicating the camera
%   motion
%               * 1:    Steady camera
%               * 2:    Handheld/shaky
%               * 3:    Panning

    % Load VLFeat
    run('../vlfeat-0.9.18/toolbox/vl_setup');
    
    nFrames = length(mov);
    [ vHeight, vWidth ] = size(mov(1).cdata);
    
    characteristics = ones(nFrames,1);
    motionDirection = zeros(10,2);
    
    Ia = single(mov(1).cdata);
    [fa,da] = vl_sift(imresize(Ia,0.5));
    for i=1:(nFrames-1)
        Ib = single(mov(i+1).cdata);
        [fb,db] = vl_sift(imresize(Ib,0.5));

        if(~ismember(i+1,cuts))
            [matches, ~] = vl_ubcmatch(da,db);
            
            motionDirection(1:9,:) = motionDirection(2:10,:);
            motionDirection(10,:) = median(fa(1:2,matches(1,:))'-fb(1:2,matches(2,:))') ./ [ vHeight, vWidth ];
            if(i > 5)
                [ muhat, sigmahat ] = normfit(motionDirection);
                muDistance = sqrt(sum(muhat.^2));
                sigmaDistance = sqrt(sum(sigmahat.^2));
                if(muDistance > sigmaDistance && muDistance > 0.03)
                    characteristics(i-5:i) = 3;
                elseif(sqrt(sum(sigmahat.^2)) > 0.01 && characteristics(i-5) ~= 3)
                    characteristics(i-5) = 2;
                end
            end
        else
            characteristics(i-5:i) = characteristics(i-6);
        end
        
        fa = fb;
        da = db;
    end


end

