function [ cuts ] = detect_cuts( mov )
%DETECT_CUTS Detect cuts in a movie
%   Input:  - mov, a movie struct
%   Output: - A list of frames AFTER a cut

addpath('helperfunctions');
lowWidth = 40;
lowHeight = 30;
dimension = lowWidth*lowHeight;
nFrames = length(mov);

%% Load low-res frames and vectorize
X = zeros(nFrames,dimension);
for frame=1:nFrames
    Ilowgray = imresize(mov(frame).cdata,[ lowHeight, lowWidth]);
    X(frame,:) = Ilowgray(:);
end

%% Calculate distance between frames
distanceVector = zeros(nFrames-1,1);
for frame=2:nFrames
   distanceVector(frame-1) =  sqrt(sum((X(frame-1,:)-X(frame,:)).^2));
end

%% Detect scene shifts
cutThreshold = 0.3;
[ cuts, ~ ] = cut_detection_frame_distance(distanceVector,cutThreshold);

end

