function [ cuts, cutsDetected ] = cut_detection_frame_distance( frameDistance, threshold )
%CUT_DETECTION_FRAME_DISTANCE Summary of this function goes here
%   Detailed explanation goes here

if(nargin < 2)
    threshold = 5/12;
end

nFrames = length(frameDistance)+1;

cuts = [];
cutsDetected = 0;
averagingLength = 3;
diffMeanVector = zeros(nFrames-1,1);
for i=2:nFrames
    % Find the frame interval (ensure that it doesn't exceed the movie
    % length
    idxAll = max(1,i-averagingLength):min(nFrames-1,i+averagingLength);
    % The frame indices without the current frame
    idxWithout = idxAll(find(idxAll ~= i));
    % Find the mean Euclidean frame distance for all frames in the interval
    meanAll = mean(frameDistance(idxAll));
    % Find the mean Euclidean frame distance excluding the current frame
    % from the interval
    meanWithout =  mean(frameDistance(idxWithout));
    % Find the difference of the means:
    diffMean = meanAll - meanWithout;
    diffMeanVector(i-1) = diffMean;
end
% Find the maximum difference:
maxMeanDiff = max(diffMeanVector);
% Detect a cut if the difference is above a given percentage of the maximum
% difference:
for i=2:nFrames
    if(diffMeanVector(i-1) > maxMeanDiff*threshold)
        cutsDetected = cutsDetected + 1;
        cuts(cutsDetected) = i+1;
    end
end
%figure,plot(diffMeanVector);

end

