function [ totalDifference ] = intensity_difference( mov, frame, threshold )
%INTENSITY_DIFFERENCE Determine the percentage of pixels changing
%significantly
%   Input:  - mov, a movie struct
%           - frame, for which the difference is calculated (must be > 1
%           and <= nFrames)
%           - threshold (optional), set threshold at which a difference is
%           significant. Default: 20

    if(nargin < 3)
        threshold = 20;
    end
    nFrames = length(mov);
    if(frame < 2 || frame > nFrames)
        error('Frame must be between 2 and the movie length');
    end
    
    probePointsXcount = 30;
    probePointsYcount = 30;
    [ vHeight, vWidth ] = size(mov(1).cdata);
    
    % Compare difference between probepoints in current and last frame
    probePointsXdistance = floor(vWidth / probePointsXcount);
    probePointsYdistance = floor(vHeight / probePointsYcount);

    probePointsX = 1:probePointsXdistance:probePointsXcount*probePointsXdistance;
    probePointsY = 1:probePointsYdistance:probePointsYcount*probePointsYdistance;

    lastFrame = mov(frame-1).cdata(probePointsY,probePointsX);
    currentFrame = mov(frame).cdata(probePointsY,probePointsX);
    diff = abs(currentFrame-lastFrame);
    % Find how many probe points show a difference greater than the
    % difference threshold
    aboveThreshold = (diff > threshold);
    totalDifference = sum(aboveThreshold(:)) / (probePointsXcount*probePointsYcount);


end

